@@ -8,6 +8,7 @@ import (
 	"io/ioutil"
 	"os"
 	"os/exec"
+	"os/signal"
 	"path/filepath"
 	"strings"
 	"time"
@@ -22,20 +23,20 @@ type RC struct {
 }
 
 // FindRC looks the RC file from the wd, up to the root
-func FindRC(wd string, config *Config) *RC {
+func FindRC(wd string, config *Config) (*RC, error) {
 	rcPath := findUp(wd, ".envrc")
 	if rcPath == "" {
-		return nil
+		return nil, nil
 	}
 
 	return RCFromPath(rcPath, config)
 }
 
 // RCFromPath inits the RC from a given path
-func RCFromPath(path string, config *Config) *RC {
+func RCFromPath(path string, config *Config) (*RC, error) {
 	hash, err := fileHash(path)
 	if err != nil {
-		return nil
+		return nil, err
 	}
 
 	allowPath := filepath.Join(config.AllowDir(), hash)
@@ -44,15 +45,15 @@ func RCFromPath(path string, config *Config) *RC {
 
 	err = times.Update(path)
 	if err != nil {
-		return nil
+		return nil, err
 	}
 
 	err = times.Update(allowPath)
 	if err != nil {
-		return nil
+		return nil, err
 	}
 
-	return &RC{path, allowPath, times, config}
+	return &RC{path, allowPath, times, config}, nil
 }
 
 // RCFromEnv inits the RC from the environment
@@ -132,13 +133,16 @@ const notAllowed = "%s is blocked. Run `direnv allow` to approve its content"
 // Load evaluates the RC file and returns the new Env or error.
 //
 // This functions is key to the implementation of direnv.
-func (rc *RC) Load(ctx context.Context, config *Config, env Env) (newEnv Env, err error) {
+func (rc *RC) Load(previousEnv Env) (newEnv Env, err error) {
+	config := rc.config
 	wd := config.WorkDir
 	direnv := config.SelfPath
-	newEnv = env.Copy()
+	newEnv = previousEnv.Copy()
 	newEnv[DIRENV_WATCHES] = rc.times.Marshal()
 	defer func() {
-		rc.RecordState(env, newEnv)
+		// Record directory changes even if load is disallowed or fails
+		newEnv[DIRENV_DIR] = "-" + filepath.Dir(rc.path)
+		newEnv[DIRENV_DIFF] = previousEnv.Diff(newEnv).Serialize()
 	}()
 
 	if !rc.Allowed() {
@@ -146,11 +150,27 @@ func (rc *RC) Load(ctx context.Context, config *Config, env Env) (newEnv Env, er
 		return
 	}
 
+	prelude := ""
+	if config.StrictEnv {
+		prelude = "set -euo pipefail && "
+	}
+
 	arg := fmt.Sprintf(
-		`eval "$("%s" stdlib)" && __main__ source_env "%s"`,
+		`%seval "$("%s" stdlib)" && __main__ source_env "%s"`,
+		prelude,
 		direnv,
 		rc.Path(),
 	)
+
+	// Allow RC loads to be canceled with SIGINT
+	ctx, cancel := context.WithCancel(context.Background())
+	c := make(chan os.Signal, 1)
+	signal.Notify(c, os.Interrupt)
+	go func() {
+		<-c
+		cancel()
+	}()
+
 	cmd := exec.CommandContext(ctx, config.BashPath, "--noprofile", "--norc", "-c", arg)
 	cmd.Dir = wd
 	cmd.Env = newEnv.ToGoEnv()
@@ -165,32 +185,20 @@ func (rc *RC) Load(ctx context.Context, config *Config, env Env) (newEnv Env, er
 		cmd.Stdin = os.Stdin
 	}
 
-	out, err := cmd.Output()
-	if err != nil {
-		return
-	}
-
-	if len(out) > 0 {
+	if out, err := cmd.Output(); err == nil && len(out) > 0 {
 		var newEnv2 Env
 		newEnv2, err = LoadEnvJSON(out)
-		if err != nil {
-			return
+		if err == nil {
+			newEnv = newEnv2
 		}
 		if newEnv2["PS1"] != "" {
 			logError("PS1 cannot be exported. For more information see https://github.com/direnv/direnv/wiki/PS1")
 		}
-		newEnv = newEnv2
 	}
 
 	return
 }
 
-// RecordState just applies the new environment
-func (rc *RC) RecordState(env Env, newEnv Env) {
-	newEnv[DIRENV_DIR] = "-" + filepath.Dir(rc.path)
-	newEnv[DIRENV_DIFF] = env.Diff(newEnv).Serialize()
-}
-
 /// Utils
 
 func rootDir(path string) string {
