@@ -17,7 +17,6 @@ func DataDir(env map[string]string, programName string) string {
 	// In theory we could also read /etc/passwd and look for the home based on
 	// the process' UID
 	return ""
-
 }
 
 // ConfigDir returns the config folder for the application
@@ -33,3 +32,15 @@ func ConfigDir(env map[string]string, programName string) string {
 	// the process' UID
 	return ""
 }
+
+// CacheDir returns the cache directory for the application
+func CacheDir(env map[string]string, programName string) string {
+	if env["XDG_CACHE_HOME"] != "" {
+		return filepath.Join(env["XDG_CONFIG_HOME"], programName)
+	} else if env["HOME"] != "" {
+		return filepath.Join(env["HOME"], ".cache", programName)
+	}
+	// In theory we could also read /etc/passwd and look for the home based on
+	// the process' UID
+	return ""
+}
